#!/usr/bin/python

# Copyright (c) 1999-2002 Red Hat, Inc.  Distributed under GPL.
#
# Author: Adrian Likins <alikins@redhat.com>
#

import sys
sys.path.append("/usr/share/rhn/")
from up2date_client import up2dateErrors
from up2date_client import up2dateAuth
from up2date_client import up2dateLog
from up2date_client import up2dateUtils
from up2date_client import up2date
from up2date_client import config
from up2date_client import rpcServer
from up2date_client import rpmUtils
from up2date_client import rollbacks
from up2date_client import rhnPackageInfo
import rpm
import time
import os

cfg = config.initUp2dateConfig()
log = up2dateLog.initLog()

# mark this module as acceptable
__rhnexport__ = [
    'update',
    'remove',
    'refresh_list'
]

global fd
def rpmCallback(what, amount, total, hdr, path):
    if what == rpm.RPMCALLBACK_INST_OPEN_FILE:
        global fd
        fileName = "%s/%s-%s-%s.%s.rpm" % (path,
                                           hdr['name'],
                                           hdr['version'],
                                           hdr['release'],
                                           hdr['arch'])
        try:
             fd = os.open(fileName, os.O_RDONLY)
        except:
            print "Error opening %s for reading" % fileName
            return -1

        return fd
    elif what == rpm.RPMCALLBACK_INST_START:
        if type(hdr) == type(""):
            print "Repackaging..."
        else:
            fileName = "%s-%s-%s.%s.rpm" % (hdr['name'],
                                            hdr['version'],
                                            hdr['release'],
                                            hdr['arch'])
    elif what == rpm.RPMCALLBACK_INST_CLOSE_FILE:
        os.close(fd)

    elif what == rpm.RPMCALLBACK_INST_PROGRESS:
        pass
    elif what == rpm.RPMCALLBACK_UNINST_STOP:
        pass

def remove(pkgList):
    """We have been told that we should remove packages"""

    if type(pkgList) != type([]):
        return (13, "Invalid arguments passed to function",{})

    log.log_debug("Called remove_packages", pkgList)

    # FIXME: figure out the error handling
    try:
        up2date.removePackages(pkgList, rpmCallback)
    except up2dateErrors.DependencyError, e:
        # just return the list of deps if we failed
        return (14, "%s" % e, {})
    except up2dateErrors.RpmRemoveError, e:
        # if we later need finer grained info, we can
        return (15, "%s" % e, {})
    except up2dateErrors.RpmRemoveSkipListError, e:
        return (16, "%s could not be removed because it was "\
                "on the skip list" % e, {})

    # if were succesful, return "4"
    return (0, "%s removed  successfully" % pkgList, {})

# stringing nay None's in dep tuples
def __cleanDeps(deps):
    clean_deps = []
    for dep in deps:
	( (name, version, release), (needsName, needsVersion),
          flags, suggested, sense) = dep	
	if flags == None:	
	    flags = ""
        if suggested == None:
	    suggested = ""
        if sense == None:
	   sense = ""
	clean_deps.append(((name,version,release),
                           (needsName, needsVersion),
                           flags, suggested, sense))
    return tuple(clean_deps)

# try to share this big old pile of exception handling:
def _catchErrors(method, name, args):
    ret = None
    try:
        ret = apply(method, args)
    except up2dateErrors.RpmError, e:
        return (18, "Failed: packages requested raised "\
                "dependency problems", {})
    except up2dateErrors.RpmInstallError, e:
        data = {}
        data['version'] = "0"
        data['name'] = "%s.package_install_failure" % name
        if e.pkg:
            data = {'package_that_failed':pkg}
        return (32, "Failed: Packages failed to install "\
                "properly: %s" % e.errmsg, data)
    except up2dateErrors.SkipListError, e:
	data = {}
	if e.pkglist:
		data = {'packages_on_skip_list': e.pkglist}
	# The version field is the version of the data struct
	data['version'] = "0"
	data['name'] = "%s.packages_on_skip_list" % name
	return (30, "Failed: Some of the packages specified were "\
                "on a skip list", data)
    except up2dateErrors.DependencyError, e:
	data = {}
	if e.deps:
	    # deps might have None's, cleanse them
	    data = {'failed_deps':__cleanDeps(e.deps)}
	data["version"] = "0"
	data["name"] = "%s.failed_deps" % name
	return (18, "Failed: packages requested raised "\
                "dependency problems", data)
    except up2dateErrors.UnsolvedDependencyError, e:
	data = {}
	if e.dep:
	   data = {'failed_deps':e.dep}
	if e.pkgs:
	   data["packages"] = e.pkgs
	   packages = up2dateUtils.pkglistToString(e.pkgs)
	else:
	   packages = ""
	data["version"] = "0"
	data["name"] = "%s.deps_on_skip_list" % name
        return(29, "The dependency %s is provided by the folowing "\
               "packages: %s, but they are not available due to client "\
               "configuration (skip lists, etc)" % (e.dep, packages), data)
    except up2dateErrors.ConflictError, e:
	if e.rc:
            # we cant marshall long ints, so make sure we cast them to ints
            try:
                tmprc = e.rc
                # old rpm always makes this an int, new rpm makes it a
                # long int, we nee
                if type(e.rc[0][1][2]) == type(1L): 
                    tmprc = [ (e.rc[0][0], (e.rc[0][1][0], e.rc[0][1][1],
                                            int(e.rc[0][1][2])) )]

            except TypeError:
                tmprc = "There was a parsing error with the rpm "\
                        "conflict message"
            except OverflowError:
                # must of be a real long int, this shouldnt ever
                # happen in this case, but dont traceback just in case
                tmprc = "There was an overflow error converting "\
                        "the return code to ints"
	    data = {'package_conflicts': tmprc}
        else:
	    data = {}
	data["version"] = "0"
	data["name"] = "%s.package_conflicts" % name
	return (26, "Failed: There was a package conflict error "\
                "with the package set: %s" % e, data)
    except up2dateErrors.FileConflictError, e:
	if e.rc: 
	    data = {'package_file_conflicts': e.rc}
	else:
	    data = {}
	data["version"] = "0"
	data["name"] = "%s.package_file_conflicts" % name
        return (27, "Failed: There was file conflict error with the "\
                "package set: %s" % e, data)
    except up2dateErrors.DependencySenseError, e:
	data = {}
	if e.sense:
		data = {'dependency_sense':e.sense}
	data["version"] = "0"
	data["name"] = "%s.dependency_sense" % name
	return (28, "There was a dependency sense error with "\
                "the sense: %s" % e.sense, data)
    except up2dateErrors.CommunicationError, e:
        return (21, "Failed: There was a communication error "\
                "talking to the server", {})
    except up2dateErrors.FileNotFoundError, e:
        data = {'missing_package_error': e}
        data["version"] = "0"
        data["name"] = "%s.file_not_found" % name
        return (31, "Failed: There was a File Not Found error. "\
                "The error was: %s" % e, data)
    except up2dateErrors.GPGVerificationUnsignedPackageError, e:
	data = {'unsigned_package':e.pkg}
	data["version"] = "0"
	data["name"] = "%s.unsigned_package" % name
        return (22, "Failed: There was a package gpg verification error. "\
                "The error was: %s" % e, data)
    except up2dateErrors.GPGVerificationError, e:
	data = {'gpg_failed_package':e.pkg}
	data["version"] = 0
	data["name"] = "%s.gpg_failed_package" % name
        return (23, "Failed: There was a package gpg verification error. "\
                "The error was: %s" % e, {})
    except up2dateErrors.GPGInstallationError:
        return (24, "Failed: gpg is not properly installed "\
                "on this machine", {})
    except up2dateErrors.GPGKeyringError:
        return (25, "Failed: The package signing key for Red Hat, Inc. "\
                "is not on the gpg keyring", {})
    except up2dateErrors.OutOfSpaceError, e:
	data = {'space_required':e.totalSize}
	data['space_available'] = e.freeDiskSpace
	data['version'] = 0
	data['name'] = "%s.out_of_space" % name
	return (33, "Failed: The was an error installing the package: "\
                "the error was: %s" % e, data)
    return ret

def update(pkgList):        
    """We have been told that we should retrieve/install packages"""

    if type(pkgList) != type([]):
        return (13, "Invalid arguments passed to function", {})

    log.log_debug("Called update_packages", pkgList)

    # if e,v,r is "", pretend it is version 0, release 0, epoch ""
    for pkg in pkgList:
        if pkg[1] == pkg[2] == pkg[3] == "":
            pkg[1] = pkg[2] = 0
            pkg[3] = ""


    # run the method, but move all the exception catching
    # elsewhere so we can share it
    ret = _catchErrors(batchRun, "packages.update", (0, pkgList))
    # little odd, if it returns a tuple, it's an error
    if type(ret) == type(()):
        return ret
 
    # bit of a hack, but slightly "better messaging"
    if cfg["retrieveOnly"]:
	return (0, "Packages sucessfully retrieved but due to "\
                "client specified options, the packages were downloaded, "\
                "but not installed.", {})
    else:
    	return (0, "Packages were installed successfully", {})


def batchRun(onlyList, pkgList):
    from up2date_client import wrapper
    ret = wrapper.batchRun(onlyList, [], 0, fromDaemon = 1,
                           actionPkgs = pkgList)
    return ret

# push again the list of rpm packages to the server
def refresh_list():
    s = rpcServer.getServer()

    log.log_debug("Called refresh_rpmlist")

    ret = None

    try:
        rhnPackageInfo.updatePackageProfile()
    except:
        print "ERROR: refreshing remote package list for System Profile"
        return (20, "Error refreshing package list", {})

    return (0, "rpmlist refreshed", {})


def main():
##    pkgList = [['alchemist-devel', '1.0.24', '4', ''],
##               ['amanda-client', '2.4.2p2', '9', ''],
##               ['amanda-devel', '2.4.2p2', '9', '']]
    pkgList = [['alchemist', '1.0.24', '4', ''],
               ['anaconda', '7.3.94', '0.200208021426', ''],
               ['anaconda-help', '7.3.94', '0.200208051123', '']]
    print remove(pkgList)

if __name__ == "__main__":
    main()
