#!/usr/bin/python

import StringIO
import packageList
import rpm
import rpmUtils
from rhpl.translate import _, N_
import headers
import up2dateUtils
import up2dateAuth
import up2dateErrors
import up2dateLog
import rpcServer
import rhnPackageInfo

try:
    from rhn import rpclib
except ImportError:
    rpclib = __import__("xmlrpclib")

headerList = headers.HeaderList()

import config
cfg = config.initUp2dateConfig()

# get a reference to the log file object
log = up2dateLog.initLog()

def showSkipListHits(skiplist):
    errmsg = _("""
    To solve all dependencies for the RPMs you have selected, The following
    packages you have marked to exclude would have to be added to the set:
    """)
    errmsg = errmsg + _("\n    %-30.30s\t%-30.30s\n") % (
        _("Package Name"), _("Reason For Skipping"))
    errmsg = errmsg + " " * 4 +  "=" * 70 + "\n"
    for i in skiplist:
        package = "%s-%s-%s" % (i[0][0], i[0][1], i[0][2])
        errmsg = errmsg + "    %-30.30s\t%-30.30s\n" % (package, i[1])
    return errmsg

def solveDep(unknowns,availList,refreshCallback=None):
    retList = []
    log.log_me("solving dep for: %s" % unknowns)
    if not refreshCallback:
        refreshCallback = 0

    s = rpcServer.getServer(refreshCallback=refreshCallback)

    try:
        retList = rpcServer.doCall(s.up2date.solveDependencies,
                                   up2dateAuth.getSystemId(),
                                   unknowns)
    except rpclib.Fault, f:
        if f.faultCode == -26:
            #raise RpmError(f.faultString + _(", depended on by %s") % unknowns)
            raise up2dateErrors.RpmError(f.faultString)
        else:
            raise up2dateErrors.CommunicationError(f.faultString)

    #FIXME: this should be cached, I dont really need to query the db
    # for this everytime
    installedPkgList = rpmUtils.getInstalledPackageList()
    installedPkgHash = {}
    for pkg in installedPkgList:
        installedPkgHash[pkg[0]] = []

    if len(retList):
	availList.sort()

        availListHash = {}
        for p in availList:
            availListHash[tuple(p[:4])] = p

        newList = []
        availListNVRE = map(lambda p: p[:4], availList)

        failedDeps = []
        pkgs = []
        for dep in retList.keys():
	    # skip the rest if we didnt get a result
            if len(retList[dep]) == 0:
                continue
            solutions = retList[dep]
            #print "-solutions: %s" % solutions
            # fixme, grab the first package that satisfies the dep
            #   but make sure we match nvre against the list of avail packages
            #   so we grab the right version of the package
            

            pkgs = []

            # if we only get one soltution, use it. No point in jumping
            # though other hoops
            if len(solutions) == 1:
                p = solutions[0]
#                print "p: %s" % p
                if availListHash.has_key(tuple(p[:4])):
                    pkgs.append(p)
#                    print "pkgs2: %s" % pkgs
                    break
            # we've got more than one possible solution, do some work
            # to figure out if I want one, some, or all of them
	    elif len(solutions) > 1:
                solutionsInstalled = []
                # step 1, see if we have any of the suggest packages that
                # solve this dep installed
                for p in solutions:
                    if installedPkgHash.has_key(p[0]):
                        solutionsInstalled.append(p)

                # we have one or more of the possible solutions
                # already installed, lets go with the newer version of
                # it
                found = 0
                if len(solutionsInstalled):
                    for p in solutionsInstalled:
                        pkgs.append(p)
                        found = 1
                    if found:
                        break
                    

                # we dont have any of possible solutions installed, pick up
                else:
                    # this is where we could do all sort of heuristics to pick
                    # best one. For now, grab the first one in the list thats
                    # available
                    for p in solutions:
                        if availListHash.has_key(tuple(p[:4])):
                            pkgs.append(p)
                            break
            else:
                # FIXME: in an ideal world, I could raise an exception here, but that will break the current gui
                pkgs.append(p)                    
                # raise UnsolvedDependencyError("Packages %s provide dep %s but are not available for install based on client config" % (pkgs,dep), dep, pkgs )

        for pkg in pkgs:
            if pkg[:4] in availListNVRE:
                newList.append(availList[availListNVRE.index(pkg[:4])])
            else:
                newList.append(pkg)
        resList = newList

    return resList


class DependencySolver:
    """This class is the engine of the dependency solver so that the external
       interface to up2date.dryRun remains readable"""
    def __init__(self, selected, availPkgs):
        self.selected = selected
        available = availPkgs
        obsoletes = rhnPackageInfo.obsoletesList()
	self.availableList = available

        # exclude the already selected ones from the available list
        xlist = map(lambda a: a[0], self.selected)
        self.available = {} 

        self.obsoletes = {}
        
        for p in obsoletes:
            key = p[5]
            if not self.obsoletes.has_key(key):
                self.obsoletes[key] = []
            self.obsoletes[key].append(p)

        # a list of excludes that might break our dependency chains
        self.excludes = []

        # make sure the ones that are skipped are not in here
        available, junk = self.__skip(available)
        for i in range(len(available)):
	    pkgName = available[i][0]
            if not pkgName in xlist:
		if not self.available.has_key(pkgName):
		    self.available[pkgName] = available[i]

        # placeholder for the database and transaction set
        self.ts = None
        
        # null callbacks for now
        self.msgCallback = self.progressCallback = self.refreshCallback = None
        
    # set the callbacks
    def set_callback(self, name, function):
        if name == "msg":
            self.msgCallback = function
        elif name == "progress":
            self.progressCallback = function
        elif name == "refresh":
            self.refreshCallback = function

    # get ready to rumble
    def setup(self):
        # prepare for the transaction list
	self.ts = rpm.TransactionSet()
	self.ts.setVSFlags(-1)

        if self.msgCallback != None:
            self.msgCallback(_("Testing package set / solving RPM "\
                               "inter-dependencies"))
        pkgset = self.selected
        self.selected = [] # the ___add will fill it back as it does ts.add
        self.__add(pkgset, check = 0)
	# JBJ: possibly dont need ordering here
#        self.ts.order()

    # refresh the screen or something
    def __refresh(self):
        if self.refreshCallback:
            self.refreshCallback()
        
    # sanitize a list of packages for the ones we have to skip
    def __skip(self, plist):
        # make sure the new packages don't include any packages
        # with packages excludes
        plist.sort()
        xlist = packageList.removeSkipPackagesFromList(plist)
        for x in xlist:
            # remove the package
            log.log_debug("Removing package", x)
	    pkgName = x[0][0]
            if self.available.has_key(pkgName): # make it unavailable too
		del self.available[pkgName]
            self.excludes.append(x)
        return plist, xlist

    # add packages to a transaction set
    def __add(self, plist, check = 0):
        log.log_debug("Candidates for the selected list:", plist)
        # get the headers
#        h_list = getHeaders(plist, refreshCallback = self.refreshCallback)
        h_list = []
        for p in plist:
            hdr = headerList[p]
            if hdr:
                h_list.append(hdr)
            else:
                log.log_debug("Could not obtain header for item:", p)

        added = [] # what packages do we add?
        
        # add them one by one
        counter = -1
        for h in h_list:
            counter = counter + 1
            if not h:
                continue
            if check:
                xlist = rpmUtils.checkHeaderForFileConfigExcludes(
                    h, plist[counter], self.ts)
                if xlist: # those packages have to be skipped too
                    log.log_debug("Got exclude list from config file check:",
                                  xlist)
                    for x in xlist:
                        log.log_debug("Skipping package %s because of "\
                                      "config file change" % (x,))
			pkgName = x[0][0]
                        if self.available.has_key(pkgName):
                            del self.available[pkgName]
                        self.excludes.append(x)
                    continue
            newp = plist[counter]
            if newp not in self.selected:
                self.selected.append(newp)
                log.log_debug("Adding to transaction set", newp)
                self.ts.addInstall(h, newp, "u")
                added.append(newp)
            if self.available.has_key(newp[0]): # no longer available
                del self.available[newp[0]]
            self.__refresh()
        return added
    
    # Solve dependencies
    def __dependencies(self, dependencies):
        if not dependencies:
            return 0
        changed = 0
        log.log_debug("Dependencies:", dependencies)
        # dont catch exceptions here, since we use them to figure out
        # if the server cant solve  a dep
        solved = solveDep(map(lambda a: a[1], dependencies),
			  self.availableList,
                          refreshCallback = self.refreshCallback)
        log.log_debug("Got back response:", solved)
        # If we got a response back, take out the packages that have been
        # already selected so we don't loop without a purpose
        for s in solved[:]:
            if s in self.selected:
                log.log_debug("Already selected:", s)
                solved.remove(s)
        # if we did not get a solution pack back, try looking at the
        # problem packages that are thorwing these dependencies.
        if not solved:
            solved = []
            snames = map(lambda a: a[0], dependencies)
	    for s in snames:
	    	if self.available.has_key(s):
                    log.log_debug("Package %s raised invalid dependency. "\
                                  "Adding %s to set" % (
                        s, self.available[s]))
                    solved.append(self.available[s])
                    changed = 1
                # look in obsoletes as well
                if self.obsoletes.has_key(s):
                    log.log_debug("Package %s raised a dep, but it is "\
                                  "obsoleted by: %s" % (s, self.obsoletes[s]))
                    # for every package that obsoletes this package,
                    # add it to the solve list if it's available
                    for p in self.obsoletes[s]:
                        if self.available.has_key(p[0]):
                            solved.append(self.available[p[0]])
                            changed = 1
        # sanitize this list
        solved, skiplist = self.__skip(solved)
        added = self.__add(solved, check = 1)
        if added:
            changed = 1
        return changed
        
    # Solve conflicts
    def __conflicts(self, conflicts):        
        if not conflicts:
            return 0
        changed = 0
        log.log_debug("Conflicts:", conflicts)
        solved = []
        for c in conflicts:
            # c should be a pair (name1, name2)
            # if one of these packages is still available, grab it
            for conflname in c:
                if self.available.has_key(conflname):
                    solved.append(self.available[conflname])
                    changed = 1
                if self.obsoletes.has_key(conflname):
                    log.log_debug("Package %s raised a dep, but it is "\
                                  "obsoleted by: %s" % (
                        conflname, self.obsoletes[conflname]))
                    # for every package that obsoletes this package, add
                    # it to the solve list if it's available
                    for p in self.obsoletes[conflname]:
                        if self.available.has_key(p[0]):
                            solved.append(self.available[p[0]])
                            changed = 1
        if solved is None and not changed:
            raise up2dateErrors.ConflictError(_(
                "Could not satisfy conflict dependencies."), conflicts)
        solved, skiplist = self.__skip(solved)
        added = self.__add(solved, check = 1)
        if added:
            changed = 1
        return changed
    
    # Format the dependency error nicely for the plain guy
    def __dep_error(self, deps):
        s = StringIO.StringIO()
        for dep in deps:
            ((name, version, release), (needsName, needsVersion),
             flags, suggested, sense) = dep
            if needsVersion:
                s.write(_("%-40.40s requires %s\n") % (
                    "%s-%s-%s" % (name, version, release),
                    "%s %s %s" % (needsName,
                                  up2dateUtils.rpmFlagsToOperator(flags),
                                  needsVersion)))
            else:
                s.write(_("%-40.40s requires %s\n") % (
                    "%s %s %s-%s" % (name,
                                     up2dateUtils.rpmFlagsToOperator(flags),
                                     version,
                                     release),
                    needsName))
        ret = s.getvalue()
        s.close()
        return ret


    def __filter_excludes_for_deps(self,deps):
        """This just filters the exclude list to any that are
        actually applicable to the failed deps"""
        # FIXME: this is probabaly fragile
        # FIXME: return the applicable and the non applicable
        #        in seperate list so we can display them as well?

        depnames = []
        for dep in deps:
            ((name, version, release), (needsName, needsVersion),
             flags, suggested, sense) = dep
            depnames.append(name)

        new_excludes = []
        for exclude in self.excludes:
            if exclude[0][0] in depnames:
                new_excludes.append(exclude)

        return new_excludes
            
    # generic dependency handler
    def process_deps(self, deps):
        if not deps:
            return 0
        conflicts = []
        dependencies = []
        last_excludes = self.excludes
        self.excludes = []
        for dep in deps:
            log.log_debug("Processing dependency", dep)
            ((name, version, release), (needsName, needsVersion),
             flags, suggested, sense) = dep
            if sense == rpm.RPMDEP_SENSE_REQUIRES:
                dependencies.append((name, needsName))
            elif sense == rpm.RPMDEP_SENSE_CONFLICTS:
                conflicts.append((name, needsName))
            else:
                raise up2dateErrors.DependencySenseError(_(
                    "Don't know how to handle dependency sense \"%s\"") %
                                                         sense, sense)
            self.__refresh()
        # now solve the dependencies...
        changed = self.__dependencies(dependencies)
        self.__refresh()
        # and the conflicts...
        changed = changed + self.__conflicts(conflicts)
        self.__refresh()
        if not changed: # blow up
            if not self.excludes: # get the last round instead
                self.excludes = last_excludes
            errmsg = ""
            applicable_excludes = self.__filter_excludes_for_deps(deps) 
            if len(applicable_excludes):
                errmsg = showSkipListHits(self.excludes)
            dep_error = self.__dep_error(deps)
            raise up2dateErrors.DependencyError(_("%s\nUnresolvable chain "\
                                                  "of dependencies:\n%s") % (
                errmsg, dep_error), deps)
        return 1
    
    # run a dependency iteration
    def solvedep(self):
        log.log_debug("Checking for dependencies")
        self.__refresh()
        deps = self.ts.check()       
        if deps:
            log.log_debug("RPM returned %d deps." % len(deps))
            # make the deps list unique:
            deps.sort()
            last = deps[-1]
            for i in range(len(deps)-2, -1, -1):
                if last == deps[i]:
                    del deps[i]
                else:
                    last = deps[i]
        ret = self.process_deps(deps)
        return ret

    # run the transaction set now to check for any other problems
    def run(self):
        log.log_debug("Running transaction (final step)...")
        self.ts.order()
        self.__refresh()
        self.ts.setFlags(rpm.RPMTRANS_FLAG_BUILD_PROBS)
        self.ts.setProbFilter(rpm.RPMTRANS_FLAG_BUILD_PROBS)
        ret = self.ts.run(rpmUtils.rpmCallback, self.progressCallback)
        self.__refresh()
        return ret
