#!/usr/bin/python
#
# $Id: up2dateAuth.py,v 1.6 2002/09/03 10:01:16 gafton Exp $

import rpcServer
import rpmSource
import config
import os
import xmlrpclib
import up2dateErrors
import up2dateUtils
import string
import up2dateLog

try:
    from rhn import rpclib
except ImportError:
    rpclib = __import__("xmlrpclib")

#grab an instance of the config class shared
cfg = config.initUp2dateConfig()
#grab an instance of the log class shared
log = up2dateLog.initLog()

loginInfo = None

def getSystemId():
    path = cfg["systemIdPath"]
    if not os.access(path, os.R_OK):
        return None
    
    f = open(path, "r")
    ret = f.read()
        
    f.close()
    return ret

# if a user has upgraded to a newer release of Red Hat but still
# has a systemid from their older release, they need to get an updated
# systemid from the RHN servers.  This takes care of that.
def maybeUpdateVersion():
    try:
        idVer = xmlrpclib.loads(getSystemId())[0][0]['os_release']
    except:
        # they may not even have a system id yet.
        return 0

    systemVer = up2dateUtils.getVersion()
    
    if idVer != systemVer:
      s = rpcServer.getServer()
    
      try:
          newSystemId = rpcServer.doCall(s.registration.upgrade_version,
                                         getSystemId(), systemVer)
      except xmlrpclib.Fault, f:
          raise up2dateErrors.CommunicationError(f.faultString)

      path = cfg["systemIdPath"]
      dir = path[:string.rfind(path, "/")]
      if not os.access(dir, os.W_OK):
          try:
              os.mkdir(dir)
          except:
              return 0
      if not os.access(dir, os.W_OK):
          return 0

      if os.access(path, os.F_OK):
          # already have systemid file there; let's back it up
          savePath = path + ".save"
          try:
              os.rename(path, savePath)
          except:
              return 0

      f = open(path, "w")
      f.write(newSystemId)
      f.close()
      try:
          os.chmod(path, 0600)
      except:
          pass



def login():

    server = rpcServer.getServer()

    systemId = getSystemId()

    if not systemId:
        return None
        
    maybeUpdateVersion()
    log.log_me("logging into up2date server")

    global loginInfo
    loginInfo = rpcServer.doCall(server.up2date.login, systemId)
    
        
    log.log_me("successfully retrieved authentication token "
               "from up2date server")

    log.log_debug("logininfo:", loginInfo)
    return loginInfo

def updateLoginInfo():
    log.log_me("updating login info")
    global loginInfo
    loginInfo = login()
    if not loginInfo:
        raise up2dateErrors.AuthenticationError("Unable to authenticate")

    rpmSource.psc.updateLoginInfo(loginInfo)
    
