#!/usr/bin/python -u
#
# Command line wrapper for Update Agent
# Copyright (c) 1999-2002 Red Hat, Inc.  Distributed under GPL.
#
# Authors:
#       Adrian Likins <alikins@redhat.com>
#       Preston Brown <pbrown@redhat.com>

import sys, os
import rpm
import string
from optik import OptionParser, Option
from types import StringType
sys.path.append("/usr/share/rhn/")
from up2date_client import config
from up2date_client import up2dateLog
from up2date_client import up2dateMessages
from up2date_client import up2dateErrors
from up2date_client import rpcServer
from up2date_client import up2dateAuth
from up2date_client import rhnPackageInfo
from up2date_client import rhnChannel
from up2date_client import packageList
from up2date_client import up2dateBatch
from up2date_client import gpgUtils
from up2date_client import rhnHardware
from up2date_client import depSolver
from up2date_client import rpmUtils


# for ssl exceptions
from OpenSSL import SSL
from OpenSSL import crypto

from rhpl.translate import _, N_

try:
    cfg = config.initUp2dateConfig()
except IOError, e:
    print up2dateMessages.rootWarningMsg
    sys.exit(1)

# not real fond of this, need to figure out a better
# way to test if we can write to the log file.
try:
    log = up2dateLog.initLog()
except IOError, e:
    print up2dateMessages.rootWarningMsg
    sys.exit(1)

from up2date_client import wrapperUtils
from up2date_client import up2date

try:
    from rhn import rpclib
except ImportError:
    rpclib = __import__("xmlrpclib")
    
try:
    from up2date_client import lilocfg
except ImportError:
    lilocfg = None

# XXX: Shouldn't these excpetions be in up2dateErrors?
class DaemonReturnError(Exception):
    pass

class CLIOptionParser(OptionParser):
    def __init__(self, from_daemon=0, *args, **kwargs):
        apply(OptionParser.__init__, (self, ) + args, kwargs)
        # We need a different way to get out for daemon mode
        self.from_daemon = from_daemon

    def print_help(self, file=None):
        OptionParser.print_help(self, file)
        
        if not file:
            file = sys.stdout
        file.write(_("""
When operating in command line mode, specifying package names as
arguments to the program will attempt to retrieve (and possibly
install, based on your configuration) those packages.  Version,
release, and architecture details will be determined by the Update
Agent automatically.
"""))


    def error(self, msg):
        self.print_usage(sys.stderr)
        sys.stderr.write("%s: error: %s\n" % (sys.argv[0], msg))
        # If ran by the daemon, raise an exception
        if self.from_daemon:
            raise DaemonReturnError
        # Otherwise, just exit
        sys.exit(1)

optionsTable = [
    Option(      "--configure",     action="store_true",
      help=_("configure Update Agent options")),
    Option("-d", "--download",      action="store_true",
      help=_("download packages only, even if configuration says to install")),
    Option("-i", "--install",       action="store_true",
      help=_("install packages, even if configuration says to download only")),
    Option(      "--src",           action="store_true",
      help=_("Download src packages as well as binary rpms")),
    Option(      "--nosrc",         action="store_true",
      help=_("Do not download src packages (overrides config setting if set)")),
    Option(      "--dbpath",        action="store", metavar="<dir>",
      help=_("Specify a path where an alternate rpm db is found")),
    Option("-f", "--force",         action="store_true",
      help=_("Force package installation, overriding package and file skip list")),
    Option(      "--justdb",        action="store_true",
      help=_("Only add packages to database, do not install to the filesystem")),
    Option("-l", "--list",          action="store_true",
      help=_("list packages available for retrieval")),
    Option(      "--dry-run",       action="store_true",
      help=_("Do everything but download and install packages (show deps needed, etc)")),
    Option("-?", "--usage",         action="store_true",
      help=_("Briefly describe the options")),
    Option("-k", "--packagedir",    action="store", metavar="<dir>",
      help=_("colon separated path of directories to look in for packages")),
    Option(      "--nosig",         action="store_true",
      help=_("do not use GPG to check package signatures")),
    Option(      "--nox",           action="store_true",
      help=_("do not attempt to use X")),
    Option("-p", "--packages",      action="store_true", 
      help=_("update packages associated with this System Profile on Red Hat Network")),
    Option(      "--hardware",      action="store_true",
      help=_("up2date hardware profile associated with this System Profile on Red Hat Network")),
    Option(      "--register",      action="store_true",
      help=_("register the system (or reregister it, if need be)")),
    Option(      "--showall",       action="store_true",
      help=_("List all packages available for download")),
    Option(      "--solvedeps",     action="store", metavar="<deps>",
      help=_("finds, downloads, and installs the packages needed to solve the list of deps")),
    Option(      "--tmpdir",        action="store", metavar="<temp-dir>",
      help=_("where to store temporary files / RPMs")),
    Option(      "--undo",          action="store_true",
      help=("undo the last package set update")),
    Option("-u", "--update",        action="store_true",
      help=_("update system with all relevant packages")),
    Option(      "--version",       action="store_true",
      help=_("show program version information")),
    Option("-v", "--verbose",       action="count",
      help=_("Show additional output")),
    Option(  "--show-channels", action="store_true", dest="show_channels",
      help=_("Show the channel names along with the package name where approriate")),
    Option(      "--whatprovides",  action="store", metavar="<deps>",
      help=_("Show the packages which solve the comma separated list of dependencies")),
    Option(      "--channel",       action="append", metavar="<channel>",
      help=_("Specify which channels to use by channel label")),
    Option(      "--gpg-flags",     action="store_true",    dest="gpg_flags",
      help=_("Show the flags that gpg will be invoked with (the keyring, etc)")),
    Option(      "--nodownload",    action="store_true",
      help=_("Do not download packages at all. For testing purposes")),
    Option(      "--uuid",          action="store", metavar="<uuid>",
      help=_("Pass in a uuid that the applet generates")),
    Option(      "--proxy",         action="store", metavar="<proxyurl>",
      help=_("Specify an http proxy to use")),
    Option(      "--proxyUser",     action="store", metavar="<proxy userid>",
      help=_("Specify a username to use with an authenticated http proxy")),
    Option(      "--proxyPassword", action="store", metavar="<proxy password>",
      help=_("Specify a password to use with an authenticated http proxy")),
    Option(      "--firstboot", action="store_true",
      help=_("Pop up in the center of the screen for Firstboot")),
    ]            
    
def printUsage(poptContext, stream = sys.stdout):
    poptContext.printUsage(stream)

def gpgWarning2(hasGui):    
    errMsg = up2dateMessages.gpgWarning2Msg 
    if hasGui:
        try:
            from up2date_client import gui
            dlg = gui.QuestionDialog( errMsg + _("\n\nInstall key?") )
            
            if dlg.getrc():
                dlg.hide()
                if gpgUtils.importGpgKeyring():
                    dlg = gui.ErrorDialog(_("""
                    Some sort of error occurred adding the Red Hat
                    GPG key to your keyring."""))
                    return 1
                else:
                    return 0
        except (ImportError, RuntimeError) :
            print _("Unable to open gui. Try `up2date --nox`")
            print errMsg
            print
            return 1
    else:
        print errMsg
        print
        return 1


def sslWarning(hasGui):
    errMsg = _("""
    Your system libraries do not support SSL (secure) connections.
    Any data that you send or receive from redhat.com will be
    transmitted in the clear.
    """)

    if hasGui:
        try:
            from up2date_client import gui
            dlg = gui.QuestionDialog(errMsg + _("Continue anyway?"))
            if not dlg.get_rc():
                return 1
            else:
                return 0
        except (ImportError, RuntimeError):
            print _("Unable to open gui. Try `up2date --nox`")
            print errMsg
            return 1
    else:
        print errMsg
        return 1



def sanityChecks(fromDaemon, hasGui):
    #  see if /var/spool/up2date exists
    if up2date.checkForStorageDir():
        wrapperUtils.warningDialog(up2dateMessages.storageDirWarningMsg,hasGui)
        if not fromDaemon:
            sys.exit(1)
        return 1

    if cfg["useGPG"] and gpgUtils.checkGPGInstallation() == 2:
        ret = gpgWarning2(hasGui)
        if ret:
            return 1
    
    if not rpcServer.hasSSL():
        if sslWarning(hasGui):
            if not fromDaemon:
                sys.exit(1)
        return 1


# the worlds ugliest function...            
def main(arglist = sys.argv, fromDaemon = 0):
    hasGui = 0
    nox = 0
    nogui = 0
    configure = 0
    continuePostConfigure = 0

    # see what we got called as 
    if os.path.basename(sys.argv[0]) == "up2date-nox":
	nox = 1

    if os.path.basename(sys.argv[0]) == "up2date-config":
        configure = 1

    if not cfg["networkSetup"]:
        configure = 1
        continuePostConfigure = 1 

    global optionsTable
    optparser = CLIOptionParser(option_list=optionsTable, 
        usage="usage: %prog [OPTIONS] [<packages>]",
        from_daemon=fromDaemon)

    try:
        (options, pkgNames) = optparser.parse_args()
    except DaemonReturnError:
        return 1

    nox = nox or options.nox
    
    try:
        if os.access("/usr/share/rhn/up2date_client/gui.py", os.R_OK):
            if os.environ["DISPLAY"] != "":
		if not nox:
                	hasGui = 1
    except:        
        pass

    if options.usage:
        optparser.print_usage()
        if fromDaemon:
            return 1
        sys.exit(0)
    if options.version:
        showVersion(hasGui)
        if fromDaemon:
            return 1
        sys.exit(0)

    # figure out the debug level
    debug = options.verbose or 0
    cfg["debug"] = cfg["debug"] + debug
    if cfg["debug"] > 2:
        # Set rpm's verbosity mode
        try:
            rpm.setVerbosity(rpm.RPMLOG_DEBUG) 
        except AttributeError:
            print "extra verbosity not supported in this version of rpm"
    
    if options.gpg_flags:
	print rpmUtils.getGPGflags()
        if fromDaemon:
            return 1
        sys.exit(0)

    if options.undo:
        ret = up2date.undo()
        sys.exit(ret)


    # set a default warning message callback, the gui version
    # should reset this
    wrapperUtils.warningCallback = wrapperUtils.printit

    registered =  up2dateAuth.getSystemId()

    if options.packages:
        if not registered:
            print up2dateMessages.needToRegister
            sys.exit(1)
        try:
            up2dateAuth.updateLoginInfo()
        except rpclib.Fault, f:
            faultError(hasGui,f.faultString)
            sys.exit(1)
        except up2dateErrors.CommunicationError, e: 
            print e 
            sys.exit(1) 
        
        print _("Updating package profile...")
        rhnPackageInfo.updatePackageProfile()
        return 0

    if options.hardware:
        if not registered:
            print up2dateMessages.needToRegister
            sys.exit(1)
        try:
            up2dateAuth.updateLoginInfo()
        except rpclib.Fault, f:
            faultError(hasGui,f.faultString)
            sys.exit(1)
        except up2dateErrors.CommunicationError, e: 
            print e 
            sys.exit(1) 
        print _("Updating hardware profile...")
        rhnHardware.updateHardware()
        return 0

    # might as well save the proxy info we get for the next useage
    save_cfg = 0
    if options.proxy:
        cfg.set("httpProxy", options.proxy)
        cfg.set("enableProxy", 1)
        save_cfg = 1        
    if options.proxyUser:
        cfg.set("proxyUser", options.proxyUser)
        cfg.set("enableProxyAuth", 1)
        save_cfg = 1        
    if options.proxyPassword:
        cfg.set("proxyPassword", options.proxyPassword)
        cfg.set("enableProxyAuth", 1)
        save_cfg = 1
    if save_cfg:
        cfg.save()
        
    depslist = options.whatprovides or []
    # XXX: what is this used for?
    if depslist:
        cfg["depslist"] = depslist
    else:
        cfg["depslist"] = []

    # Easy mapping of flags
    # Syntax: 
    #  l[0] == name of the option
    #  l[1] == name in the config object
    #  l[2] == value to set in the config object
    flags = [
        # XXX
        ["download",        "retrieveOnly",     1],
        ["install",         "retrieveOnly",     0],
        ["nodownload",      "noDownload",       1],
        ["force",           "forceInstall",     1],
        # XXX
        ["src",             "retrieveSource",   1],
        ["nosrc",           "retrieveSource",   0],
        ["tmpdir",          "storageDir",       options.tmpdir],
        ["nosig",           "useGPG",           0],
        ["dbpath",          "dbpath",           options.dbpath],
        ["justdb",          "justdb",           1],
        ["show_channels",   "showChannels",     1],
        ['uuid',            "uuid",             options.uuid]
#        ["gpg_flags",       "gpg-flags",         1],
       
        
    ]

    for flag in flags:
        if getattr(options, flag[0]):
            cfg[flag[1]] = flag[2]


    packagedir = options.packagedir
    if packagedir:
        packagedir = os.path.normpath(
            os.path.abspath(os.path.expanduser(packagedir)))
        cfg["packageDir"] = packagedir

    # Done with the flags processing

    tempchannels = options.channel or []
    if len(tempchannels):
        if not registered:
            print up2dateMessages.needToRegister
            sys.exit(1)
        try:
            up2dateAuth.updateLoginInfo()
        except rpclib.Fault, f:
            faultError(hasGui,f.faultString)
            sys.exit(1)
        except up2dateErrors.CommunicationError, e: 
            print e 
            sys.exit(1) 
        rhnChannel.updateChannels(tempchannels)
	
    if options.showall:
        if not registered:
            print up2dateMessages.needToRegister
            sys.exit(1)
        # FIXME: hack, should set a variable here and then run it later
        try:
            up2dateAuth.updateLoginInfo()
        except rpclib.Fault, f:
            faultError(hasGui,f.faultString)
            sys.exit(1)
        except up2dateErrors.CommunicationError, e: 
            print e 
            sys.exit(1) 
        printList(up2date.getAvailablePackageList())
        return 0

    fullUpdate = options.update
    configure = configure or options.configure

    # see if were running as root, before running configure
    if os.geteuid() != 0:
        wrapperUtils.warningDialog(up2dateMessages.rootWarningMsg, hasGui)
        if not fromDaemon:
            sys.exit(1)

    if configure:
        if hasGui:
            try:
                from up2date_client import configdlg
            except:
                hasGui = 0
            if hasGui:
                ret = configdlg.main(options.firstboot)
                if fromDaemon:
                    return 0
                elif continuePostConfigure and ret:
                    cfg.set("networkSetup", 1)
                    cfg.save()
                else:
                    sys.exit(0)
        else:
            from up2date_client import text_config
            text_config.main()
            if fromDaemon:
                return 1
            elif continuePostConfigure:
                cfg.set("networkSetup", 1)
                cfg.save()
            else:
                sys.exit(1)

    # run all the sanity checks (gpg key, paths, etc)
    ret = sanityChecks(fromDaemon, hasGui)
    if ret:
        return ret

    
    # force a registration if we specify it on the commandline
    if options.register:
        registered = 0
        
    if registered:
        try:
            up2dateAuth.updateLoginInfo()
        except rpclib.Fault, f:
            faultError(hasGui,f.faultString)
            sys.exit(1)
        
        except up2dateErrors.CommunicationError, e: 
            print e 
            sys.exit(1) 

#FIXME

    if len(cfg["depslist"]):
       return whatprovides(cfg["depslist"])


#FIXME: this stuff needs to move to post registration
    if options.solvedeps:
        # maybe this will handle comboa cases of packages and deps
        # specified?? NEEDSTEST
        extraPackages = whatprovides(options.solvedeps, 1)
        if extraPackages:
            pkgNames = pkgNames + extraPackages
        else:
            print _("No packages were found that satisfy those dependencies.")
            sys.exit(1)

    if cfg["showChannels"]:
        channels = up2dateAuth.loginInfo['X-RHN-Auth-Channels']
        for channel in channels:
            print "%s" % channel[0]
        # nothing else that uses showChannel follows, so we can exit here
        sys.exit(0) 


    if not registered:
        if hasGui:
	    try:
            	from up2date_client import gui
                nogui = 0
	    except RuntimeError:
                # if we fail to initialize the interface, fall back to tui
                nogui = 1
                
 	    if not nogui:
                wrapperUtils.warningCallback = gui.errorWindow
                if not registered:
                    gui.forceRegister = 1
            	gui.main(options.firstboot)
            	sys.exit(0)
        else:
            # call the registration client first...
            from up2date_client import tui
            tui.main()


    # if we dont have loginingo because we needed
    # to register, login now
    if not up2dateAuth.loginInfo:
        try:
            up2dateAuth.updateLoginInfo()
        except rpclib.Fault, f:
            faultError(hasGui,f.faultString)
            sys.exit(1)
        except up2dateErrors.AuthenticationError, e:
            sys.exit(1)
            
        except up2dateErrors.CommunicationError, e: 
            print e 
            sys.exit(1) 


    if options.list or len(pkgNames) or fullUpdate or options.dry_run:

        # continue on in batch mode
        
        if fromDaemon:
            return batchRun(options.list, pkgNames, fullUpdate, fromDaemon)
        try:
            sys.exit(batchRun(options.list, pkgNames,
			      fullUpdate, dryRun=options.dry_run))
        except up2dateErrors.GPGVerificationError, e:
            # might as well be annoying and loud about it
            print
            print "************ GPG VERIFICATION ERROR ****************"
            print e
            print "****************************************************"
            print
            sys.exit(1)
        except up2dateErrors.FileNotFoundError, e:
            print  e.errmsg
            sys.exit(1)
        except up2dateErrors.GPGVerificationUnsignedPackageError,e:
            print e, "Aborting..."
            sys.exit(1)
        except up2dateErrors.CommunicationError, e:
            print _("There was a fatal error communicating with the server. "\
                    "The message was:\n") + e.errmsg
            sys.exit(1)
        except up2dateErrors.RpmError, e:
            print _("There was a fatal RPM error. "\
                    "The message was:\n") + e.errmsg
            sys.exit(1)
        except up2dateErrors.RpmInstallError, e:
            print _("There was a fatal RPM install error. "\
                    "The message was:\n") + e.errmsg
            sys.exit(1)
        except up2dateErrors.DependencyError, e:
            print _("There was a package dependency problem. "\
                    "The message was:\n") + e.errmsg
            sys.exit(1)
        except up2dateErrors.UnsolvedDependencyError, e:
            print _("There was a package dependency problem. "\
                    "The message was:\n") + e.errmsg
            sys.exit(1)
        except up2dateErrors.OutOfSpaceError, e:
            print _("There was an error installing the packages. "\
                    "The message was:\n") + e.errmsg
            sys.exit(1)
        except up2dateErrors.ConflictError, e:
            print e
            sys.exit(1)
            
    else: # no --list
        if hasGui:
	    try:
            	from up2date_client import gui
                nogui = 0
	    except RuntimeError:
                # if we fail to initialize the interface, fall back to tui
                nogui = 1
                
 	    if not nogui:
                wrapperUtils.warningCallback = gui.errorWindow
            	gui.main(options.firstboot)	
            	sys.exit(0)
                
        print _("No interactive mode available")
        print _("Please specify either -l, -u, --nox, or package names "\
                "as command line arguments.")
        optparser.print_help()
        if fromDaemon:
            return 1
        else:
            sys.exit(1)
        
def ReportError(*args):
    sys.stderr.write("%s\n" % string.join(map(str, args)))

def showVersion(hasGui):
    if hasGui:
        try:
            from up2date_client import gui
            gui.showAbout()
        except:
            print _("Unable to open gui. Try specifying \"--nox\" "\
                    "as an option")
    else:
        print "Red Hat Update Agent v%s" % up2date.version()
        print "Copyright (C) 1999-2002 Red Hat, Inc."
        print _("Licensed under terms of the GPL.")


def faultError(hasGui,msg):
    errMsg = msg
    
    if hasGui:
        try:
            from up2date_client import gui
            gui.errorWindow(errMsg)
        except:
            print _("Unable to open gui. Try `up2date --nox`")
            print errMsg
    else:
        print errMsg

def whatprovides(depslist, returnlist=None):
    tmp_list = string.split(depslist, ",")
    deps_list=[]
    availlist = up2date.getAvailablePackageList()
    # a little text munging to make this commandline friendlier
    for i in tmp_list:
        i=string.strip(i)
        deps_list.append(i)
    try:
        provides = depSolver.solveDep(deps_list,availlist)
    except up2dateErrors.DependencyError:
        print "Unable to solve deps for  %s" % deps_list       
        return None
    if returnlist:
        pkgnames = []
        for provide in provides:
            pkgnames.append(provide[0])
        return pkgnames
    pkgDict = up2date.avail_package_dict()
    for provide in provides:
        pkgString = "%s-%s-%s" % (provide[0], provide[1],provide[2])
        if cfg["showChannels"]:
            # print the channel name
            channelName = pkgDict["%s-%s-%s.%s" % (
                provide[0], provide[1],provide[2], provide[4])][6]
            print "%-40s%-30s" % (pkgString, channelName)
        else:
            print "%s" % pkgString


def printList(packageList):
    packageList.sort()
    pkgDict = up2date.avail_package_dict()
    for pkg in packageList:
        pkgString = "%s-%s-%s" % (pkg[0], pkg[1],pkg[2])
        if cfg["showChannels"]:
            clabel = "%s-%s-%s.%s" % (pkg[0],pkg[1],pkg[2],pkg[4])
            channelName = pkgDict[clabel][6]
            print "%-40s%-30s" % (pkgString, channelName)
        else:
            print "%s" % (pkgString)

    
def batchRun(onlyList, pkgNames, fullUpdate = 0,
             fromDaemon = 0, actionPkgs=None,
	     dryRun=None):

    rpmCallback = wrapperUtils.RpmCallback()

    batch = up2dateBatch.BatchRun()
    batch.onlyList = onlyList
    batch.dryRunFlag = dryRun

    # quiet mode for rhn_check
    if fromDaemon:
        batch.printCallback = None
        batch.percentCallback = None
    else:
        batch.printCallback = wrapperUtils.printit
        batch.percentCallback = wrapperUtils.percent

    batch.listOfPkgNames = pkgNames
    batch.onlyList = onlyList
    batch.fromDaemon = fromDaemon
    batch.actionPkgs = actionPkgs
    batch.rpmCallback = rpmCallback.callback


    batch.run()

    # cases...
    # case 1: I specify a package to install, the package is availa
    #         I want the dep list, and the warning if it's found, but
    #         not the info about packages that were skipped unless it was
    #         one of the cmdline packages or it's deps

    # case 2: a full update... I want all the info
    #
    # case 3: actions... not sure...


    if len(batch.skippedPackages):
        wrapperUtils.printSkippedPackages(batch.skippedPackages)

    if not batch.packagesToInstall:
        print _("None of the packages you requested were found, "\
                "or they are already updated.")
	# if we got no updates, but packages we skipped, raise an
	# exception indicating that

        # FIXME: insert code here to determine if packages were on
        # package skip, file skip, or skipped because of config
        # stuff... probabaly need to add code to PackageList to expose
        # that info individually and then make BatchRun know that...
	if fromDaemon:
	    if len(batch.skippedPackages):
	        raise up2dateErrors.SkipListError(
                    "The following packages were on your skip list "\
                    "and could not be installed: %s" % batch.skippedPackages,
                    batch.skippedPackages)

##	    if len(skippedPackagesFileConfigSkipList):
##		raise up2dateErrors.FileConfigSkipListError(
##                    "The following packages were on your skip list "\
##                    "and could not be installed: %s" %
##                    skippedPackagesFileConfigSkipList,
##                    skippedPackagesFileConfigSkipList)

	return 1

    #FIXME:? should batch run do this itself?
    if len(batch.depPackages):
        wrapperUtils.printDepPackages(batch.depPackages)


    #FIXME: maybe we should split Batch.run() into Batch.setup(),
    # Batch.fetch(), and Batch.install() so we can handle the errors
    # better?
    
    if batch.kernelsToInstall and lilocfg:
        try:
            up2date.installBootLoader(batch.kernelsToInstall)
        except (lilocfg.LiloConfError, lilocfg.LiloInstallError),  info:
            print "%s" % info
            return 1

    # we're fine up to this point...
    return 0
            
        
if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        print "\nAborted."
        sys.exit(1)
    except OSError, e:
        print _("An unexpected OS error occurred: %s") % e
        sys.exit(1)
    except IOError, e:
        print _("There was some sort of I/O error: %s") % e
        sys.exit(1)
    except SSL.Error, e:
        print _("There was an SSL error: %s") % e
        sys.exit(1)
    except crypto.Error,e:
        print _("There was a SSL crypto error: %s") % e
